package com.safeway.pricing.safeway.controller;


import com.safeway.pricing.safeway.dto.ItemDetailsDTO;
import com.safeway.pricing.safeway.dto.SaleDetailsDTO;
import com.safeway.pricing.safeway.dto.StoreGroupDTO;
import com.safeway.pricing.safeway.entity.ItemDetails;
import com.safeway.pricing.safeway.entity.PriceAreaDetails;
import com.safeway.pricing.safeway.entity.PriceDetails;
import com.safeway.pricing.safeway.entity.StoreDetails;
import com.safeway.pricing.safeway.service.PricingService;
import com.safeway.pricing.safeway.util.AppUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.publisher.Sinks;
import rx.Observable;
import rx.Observer;

import java.util.List;

@RestController
@RequestMapping("/safeway")
public class PricingController {

    @Autowired
    private PricingService pricingService;


    @GetMapping(value="/storedetails/{storeName}",produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<StoreDetails> getStoreDetails(@PathVariable String storeName){
        pricingService.getStoreDetails(storeName);
        return pricingService.getStoreDetailsMany().asFlux();
    }


    @GetMapping(value="/pricingdetails")
    public Flux<PriceDetails> getPricingDetails(){
        return pricingService.getPricingDetails();
    }

    @PostMapping(value="/addstoredetails")
    public Mono<StoreDetails> addStoreDetails(@RequestBody StoreDetails storeDetails){
        return pricingService.addStoreDetails(storeDetails);
    }

    @GetMapping("/allItemDetails")
    public Flux<ItemDetailsDTO> allItemDetails() {
        return this.pricingService.getItemDetails();
    }
    
    @PostMapping(value="/saveItemDetails")
    public ItemDetails saveItemDetails(@RequestBody ItemDetailsDTO itemDetailsDTO){
        return pricingService.saveItemDetails(itemDetailsDTO);
    }


    @GetMapping("/allStoreGroup")
    public Flux<StoreGroupDTO> allStoreGroup() {
        return this.pricingService.getStoreGroups();
    }


    @GetMapping("/allSaleDetails")
    public Flux<SaleDetailsDTO> allSaleDetails() {
        return this.pricingService.getSalesDetails();
    }
    
    @GetMapping("/group1")
	@PreAuthorize("hasRole('ROLE_group1')")
	public String getDetails(@AuthenticationPrincipal(expression = "claims['name']") String name) {
		
		return "Hellow User "+name;
	}

	
}
