package com.safeway.pricing.safeway.service;

import com.safeway.pricing.safeway.dto.ItemDetailsDTO;
import com.safeway.pricing.safeway.dto.SaleDetailsDTO;
import com.safeway.pricing.safeway.dto.StoreGroupDTO;
import com.safeway.pricing.safeway.entity.ItemDetails;
import com.safeway.pricing.safeway.entity.PriceAreaDetails;
import com.safeway.pricing.safeway.entity.PriceDetails;
import com.safeway.pricing.safeway.entity.StoreDetails;
import com.safeway.pricing.safeway.modelmapper.ItemDetailsModelMapper;
import com.safeway.pricing.safeway.modelmapper.SaleDetailsModelMapper;
import com.safeway.pricing.safeway.modelmapper.StoreGroupModelMapper;
import com.safeway.pricing.safeway.repository.*;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.core.publisher.Sinks;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Data
public class PricingService {

    private Sinks.Many<StoreDetails> storeDetailsMany=null;

    @Autowired
    private PricingServiceData pricingServiceData;
    @Autowired
    private ItemDetailsRepository repository;


    @Autowired
    private ItemDetailsModelMapper itemDetailsModelMapper;

    @Autowired
    private SaleDetailsRepository saleDetailsRepository;

    @Autowired
    private SaleDetailsModelMapper saleDetailsModelMapper;

    @Autowired
    private StoreGroupRepository storeGroupRepository;

    @Autowired
    private StoreGroupModelMapper storeGroupModelMapper;

    public void createSink(){
        storeDetailsMany=Sinks.many().replay().all();
    }
    public void getStoreDetails(String storeName) {
        createSink();
        List<StoreDetails> storeDetailsList=pricingServiceData.getStoreDetails();
        List<StoreDetails> storeDetailsList1=storeDetailsList.stream().filter(storeDetails -> storeDetails.getStoreName().equalsIgnoreCase(storeName)).collect(Collectors.toList());
        Flux.fromIterable(storeDetailsList1).subscribe(storeDetails -> storeDetailsMany.tryEmitNext(storeDetails));
    }

    public Flux<PriceDetails> getPricingDetails(){
        List<PriceDetails> list=pricingServiceData.getPriceDetails();
        return Flux.fromIterable(list);
    }

    public Mono<PriceAreaDetails> getPriceAreaDetails() {
        PriceAreaDetails priceAreaDetails=pricingServiceData.getPriceAreaDetails();
        return Mono.just(priceAreaDetails);
    }

    public Mono<StoreDetails> addStoreDetails(StoreDetails storeDetails){
        Mono<StoreDetails> storeDetailsMono;
             if(storeDetailsMany==null){
                storeDetailsMono = pricingServiceData.addStore(storeDetails);
             }else
                storeDetailsMono = pricingServiceData.addStore(storeDetails).doOnNext(storeDetails1 -> storeDetailsMany.tryEmitNext(storeDetails1));

         return storeDetailsMono;
    }
    public Flux<ItemDetailsDTO> getItemDetails() {
        return Flux.fromIterable(this.repository.findAll())
                .map(dto -> itemDetailsModelMapper.itemDetailsDTO(dto));
    }
    
    public ItemDetails saveItemDetails(ItemDetailsDTO itemDetailsDTO) {
    	ItemDetails itemDetails =itemDetailsModelMapper.itemDetailsEntity(itemDetailsDTO);
		return this.repository.save(itemDetails);
	}

    public Flux<StoreGroupDTO> getStoreGroups() {
        return Flux.fromIterable(this.storeGroupRepository.findAll())
                .map(dto -> storeGroupModelMapper.storeGroupDTOtoreGroup(dto));
    }

    public Flux<SaleDetailsDTO> getSalesDetails() {
        return Flux.fromIterable(this.saleDetailsRepository.findAll()).map(dto->saleDetailsModelMapper.saleDetailsDTO(dto));
    }
	
}
